import subprocess
import re
import os
import sys
from typing import List, Optional

# ANSI 颜色代码
class Colors:
    RED = '\033[91m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    BLUE = '\033[94m'
    MAGENTA = '\033[95m'
    CYAN = '\033[96m'
    RESET = '\033[0m'

def find_requirements_file():
    """从当前目录向上查找 requirements.txt"""
    current_dir = os.getcwd()
    while True:
        file_path = os.path.join(current_dir, 'requirements.txt')
        if os.path.isfile(file_path):
            return file_path
        parent_dir = os.path.dirname(current_dir)
        if parent_dir == current_dir:
            print(f"{Colors.RED}错误：未找到 requirements.txt 文件。{Colors.RESET}")
            return None
        current_dir = parent_dir

def read_requirements(file_path: str) -> List[str]:
    """读取 requirements.txt，自动处理编码问题（含 chardet 自动安装）"""
    try:
        with open(file_path, 'rb') as f:
            content = f.read()
            try:
                return [line.strip() for line in content.decode('utf-8').splitlines()
                        if line.strip() and not line.startswith('#')]
            except UnicodeDecodeError:
                try:
                    import chardet
                except ModuleNotFoundError:
                    print(f"{Colors.YELLOW}检测到缺少 chardet，正在自动安装...{Colors.RESET}")
                    process = subprocess.run(
                        [sys.executable, "-m", "pip", "install", "chardet"],
                        check=False,
                        text=True
                    )
                    if process.returncode != 0:
                        print(f"{Colors.RED}错误：无法安装 chardet，请手动安装后重试。{Colors.RESET}")
                        return []
                    import chardet

                encoding = chardet.detect(content)['encoding'] or 'utf-8'
                return [line.strip() for line in content.decode(encoding).splitlines()
                        if line.strip() and not line.startswith('#')]
    except FileNotFoundError:
        print(f"{Colors.RED}错误：未找到 {file_path} 文件。{Colors.RESET}")
        return []

def parse_requirement(requirement: str) -> Optional[str]:
    match = re.match(
        r'^([a-zA-Z0-9][a-zA-Z0-9\-_.]+)\s*(?:==|>=|<=|!=|<|>|~=)?\s*([^;\s]*)',
        requirement
    )
    if match:
        return match.group(1)
    return None

def uninstall_module(package: str) -> bool:
    print(f"{Colors.CYAN}正在卸载: {package}{Colors.RESET}")
    try:
        process = subprocess.run(
            ["pip", "uninstall", "-y", package],
            check=False,
            text=True
        )
        return process.returncode == 0
    except Exception:
        return False

def main():
    requirements_file = find_requirements_file()
    if not requirements_file:
        return

    requirements = read_requirements(requirements_file)

    uninstalled_count = 0
    skipped = []

    print(f"{Colors.YELLOW}\n=== 开始卸载依赖包 ==={Colors.RESET}")

    for requirement in requirements:
        package = parse_requirement(requirement)
        if not package:
            skipped.append(requirement)
            continue

        if uninstall_module(package):
            uninstalled_count += 1
        else:
            skipped.append(requirement)
            print(f"{Colors.RED}× 卸载失败: {package}{Colors.RESET}")

    print(f"\n{Colors.YELLOW}=== 卸载结果总结 ==={Colors.RESET}")
    print(f"{Colors.GREEN}成功卸载: {uninstalled_count} 个包{Colors.RESET}")

    if skipped:
        print(f"\n{Colors.RED}卸载失败或无效包: {len(skipped)} 个{Colors.RESET}")
        for req in skipped:
            print(f"  {req}")

    print(f"\n{Colors.YELLOW}=== 卸载完成 ==={Colors.RESET}")

if __name__ == "__main__":
    main()
    